#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

uniform float _ShineRotate;
uniform float _ShineLocation;
uniform float _ShineWidth;
uniform float _ShineOpacity;
uniform vec4 _ShineColor;

uniform int _ShineMetallic; //0:Plastic, 1:Metallic?
uniform int _ShineType; // 0:Linear, 1:Radial

#define PI 3.14159

vec3 Vibrance(vec3 In, float amount)
{
    vec4 col = vec4(In,1.0);
    vec4 coeff = vec4(0.299,0.587,0.114, 0.);
    float lum = dot(col, coeff);
    vec4 mask = (col - vec4(lum));
    mask = clamp(mask, 0.0, 1.0);
    float lumMask = dot(coeff, mask);
    lumMask = 1.0 - lumMask;
    col = mix(vec4(lum), col, 1.0 + amount * lumMask);
    return col.rgb;
}

vec3 ShineMetallic(vec3 color, float mask)
{
    color.rgb += mask*.5;
    float mask1 = clamp(mask, 0., 1.);
    color.rgb = Vibrance(color.rgb, mask1 * 2.);
    return color.rgb;
}

void main(){
    vec4 col = texture2D(CC_Texture0, v_texCoord);
    if(col.a > 0.95)
    {
        vec2 uvShine = v_texCoord;
        float cosAngle = cos(_ShineRotate);
        float sinAngle = sin(_ShineRotate);
        mat2 rot = mat2(cosAngle, -sinAngle, sinAngle, cosAngle);
        uvShine -= vec2(0.5, 0.5);
        uvShine = uvShine * rot;
        uvShine += vec2(0.5, 0.5);

        float currentDistanceProjection = (uvShine.x + uvShine.y) / 2.;
        float whitePower = 1. - (abs(currentDistanceProjection - _ShineLocation) / _ShineWidth);
        
        float mask = 0.0;
        
        if (_ShineType == 0)
        {
            mask = col.a * whitePower * _ShineOpacity * max(sign(currentDistanceProjection - (_ShineLocation - _ShineWidth)), 0.0)
            * max(sign((_ShineLocation + _ShineWidth) - currentDistanceProjection), 0.0);
        }
        
        if (_ShineType == 1)
        {
            float angle = atan(uvShine.s-0.5, uvShine.t-0.5);
            angle = 1.0 - smoothstep(PI, PI -0.5, angle);
            mask = angle * _ShineOpacity;
        }
        
        col.rgb += mask * _ShineColor.rgb * col.a;
        
        if(_ShineMetallic == 1)
        {
            col.rgb = ShineMetallic(col.rgb, mask);
        }
        
    }
    gl_FragColor = col;
        //gl_FragColor = vec4(vec3(mask),1.0);

}
