#ifdef GL_ES
precision highp float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

uniform vec2 resolution;
uniform float radius;

void main(void)
{
    vec4 texColor = texture2D(CC_Texture0, v_texCoord);
    
    if(texColor.a > 0.1)
    {
        vec2 unit = 1.0 / resolution.xy;
        
        float i;
        
        vec2 dist = vec2(0.0);
        
        for(i=1.0;i<radius;i+=0.2)
        {
            if(texture2D(CC_Texture0, v_texCoord + vec2(i*unit.x,0)).a == 0.0)
            {
                dist.x = i;
                break;
            }
            
            if(texture2D(CC_Texture0, v_texCoord + vec2(-i*unit.x,0)).a == 0.0)
            {
                dist.x = -i;
                break;
            }
        }
        
        for(i=1.0;i<radius;i+=0.2)
        {
            if(texture2D(CC_Texture0, v_texCoord + vec2(0,i*unit.y)).a == 0.0)
            {
                dist.y = i;
                break;
            }
            
            if(texture2D(CC_Texture0, v_texCoord + vec2(0,-i*unit.y)).a == 0.0)
            {
                dist.y = -i;
                break;
            }
        }
        
        if(dist.x != 0.0 && dist.y != 0.0)
        {
            vec2 distAbs = abs(dist);
            
            if((radius - distAbs.x) * (radius - distAbs.x) + (radius-distAbs.y) * (radius - distAbs.y) > radius * radius)
            {
                gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
                return;
            }
            
            vec2 current = v_texCoord / unit;
            
            vec2 center;
            
            if(dist.x > 0.0)
            {
                center.x = current.x  - (radius - dist.x);
            }
            else
            {
                center.x = current.x  + (radius + dist.x);
            }
            
            if(dist.y > 0.0)
            {
                center.y = current.y  - (radius - dist.y);
            }
            else
            {
                center.y = current.y  + (radius + dist.y);
            }
            
            dist = current - center;
            
            i = length(dist); // i: distance from center
            
            if(i > radius * 0.1)
            {
                float offsetFromEdge = radius - i;
                
                distAbs = abs(dist);
                
                if(current.x > center.x)
                {
                    if(current.y > center.y)
                    {
                        if(distAbs.x > distAbs.y)
                        {
                            gl_FragColor = texture2D(CC_Texture0, unit * vec2(center.x + radius - offsetFromEdge, center.y + distAbs.y));
                        }
                        else
                        {
                            gl_FragColor = texture2D(CC_Texture0, unit * vec2(center.x + distAbs.x, center.y + radius - offsetFromEdge));
                        }
                    }
                    else
                    {
                        if(distAbs.x > distAbs.y)
                        {
                            gl_FragColor = texture2D(CC_Texture0, unit * vec2(center.x + radius - offsetFromEdge, center.y - distAbs.y));
                        }
                        else
                        {
                            gl_FragColor = texture2D(CC_Texture0, unit * vec2(center.x + distAbs.x, center.y - radius + offsetFromEdge));
                        }
                    }
                }
                else
                {
                    if(current.y > center.y)
                    {
                        if(distAbs.x > distAbs.y)
                        {
                            gl_FragColor = texture2D(CC_Texture0, unit * vec2(center.x - radius + offsetFromEdge, center.y + distAbs.y));
                        }
                        else
                        {
                            gl_FragColor = texture2D(CC_Texture0, unit * vec2(center.x - distAbs.x, center.y + radius - offsetFromEdge));
                        }
                    }
                    else
                    {
                        if(distAbs.x > distAbs.y)
                        {
                            gl_FragColor = texture2D(CC_Texture0, unit * vec2(center.x - radius + offsetFromEdge, center.y - distAbs.y));
                        }
                        else
                        {
                            gl_FragColor = texture2D(CC_Texture0, unit * vec2(center.x - distAbs.x, center.y - radius + offsetFromEdge));
                        }
                    }
                }
                
                return;
            }
        }
    }
    
    gl_FragColor = texColor;
    
}
